import torch
from torch_geometric.nn import GATConv


def test_gat_conv():
    in_channels, out_channels = (16, 32)
    edge_index = torch.tensor([[0, 0, 0, 1, 2, 3], [1, 2, 3, 0, 0, 0]])
    num_nodes = edge_index.max().item() + 1
    x = torch.randn((num_nodes, in_channels))

    conv = GATConv(in_channels, out_channels, heads=2, dropout=0.5)
    assert conv.__repr__() == 'GATConv(16, 32, heads=2)'
    assert conv(x, edge_index).size() == (num_nodes, 2 * out_channels)
    assert conv((x, x), edge_index).size() == (num_nodes, 2 * out_channels)

    out = conv(x, edge_index, return_attention_weights=True)
    assert out[0].size() == (num_nodes, 2 * out_channels)
    assert out[1][1].size() == (edge_index.size(1) + num_nodes, 2)
    assert conv.__alpha__ is None

    conv = GATConv(in_channels, out_channels, heads=2, concat=False)
    assert conv(x, edge_index).size() == (num_nodes, out_channels)
    assert conv((x, x), edge_index).size() == (num_nodes, out_channels)

    out = conv(x, edge_index, return_attention_weights=True)
    assert out[0].size() == (num_nodes, out_channels)
    assert out[1][1].size() == (edge_index.size(1) + num_nodes, 2)
    assert conv.__alpha__ is None
